/** @file runtime/token.h
 * Definitions for managing tokens in the unification process
 */

/** declare a token with bindable variables
 * @param type	type of the associated input place
 * @param num	number of the token
 */
#define TOKEN(type,num)				\
static struct {					\
  card_t count;					\
  card_t reserved;				\
  struct tree##type* m;				\
  struct tree##type* i;				\
} token##num

/** declare a concrete token
 * @param type	type of the associated input place
 * @param num	number of the token
 */
#define TOKENC(type,num)			\
static struct {					\
  card_t count;					\
  card_t reserved;				\
  struct tree##type* m;				\
  struct tree##type* i;				\
  t##type item;					\
} token##num

/** declare the only token contained in a place
 * @param type	type of the associated input place
 * @param num	number of the token
 */
#define TOKEN1(type,num)			\
static struct {					\
  card_t count;					\
  t##type* item;				\
} token##num

/** initialize a token with bindable variables
 * @param num	number of the token
 * @param place	the associated input place
 */
#define INIT_TOKEN(num,place)			\
do {						\
  token##num.reserved = 0;			\
  token##num.m = place; FIRST (token##num.m);	\
  while (token##num.m && !token##num.m->count)	\
    NEXT (token##num.m);			\
  token##num.i = token##num.m;			\
} while (0)
/** initialize a concrete token
 * @param num	number of the token
 * @param place	the associated input place
 */
#define INIT_TOKENC(num,place) (token##num.reserved = 0, token##num.m = place)

/** reserve a token
 * @param num	number of the token
 */
#define RESERVE_TOKEN(num)						\
(token##num.i->count -= (token##num.reserved = token##num.count))	\
/** release a token
 * @param num	number of the token
 */
#define RELEASE_TOKEN(num)						\
((token##num.i->count += token##num.reserved), token##num.reserved = 0)
/** determine whether there are enough tokens in a place
 * @param num	number of the token
 */
#define ENOUGH_TOKEN(num) (token##num.count <= token##num.i->count)
/** step to the next token
 * @param num	number of the token
 */
#define NEXT_TOKEN(num) NEXT(token##num.i)
/** find a token
 * @param num	number of the token
 * @param t	type of the token
 */
#define FIND_TOKEN(num,t) \
((token##num.i=find##t (token##num.m, &token##num.item)) && ENOUGH_TOKEN (num))
