// Global marking (PlaceMarking for each place) -*- c++ -*-

#ifdef __GNUC__
# pragma implementation
#endif // __GNUC__
#include "GlobalMarking.h"
#include "Net.h"
#include "Place.h"

#include <cassert>

/** @file GlobalMarking.C
 * Assignment of markings for each place
 */

GlobalMarking::GlobalMarking (const class Net& net) :
#ifndef NDEBUG
  myNet (net),
#endif // NDEBUG
  myErroneous (false),
  mySize (net.getNumPlaces ()),
  myPlaceMarkings (NULL)
{
  if (mySize)
    myPlaceMarkings = new class PlaceMarking[mySize];
  for (unsigned i = 0; i < mySize; i++)
    myPlaceMarkings[i].setPlace (&net.getPlace (i));
}

GlobalMarking::GlobalMarking (const class GlobalMarking& old) :
#ifndef NDEBUG
  myNet (old.myNet),
#endif // NDEBUG
  myErroneous (old.myErroneous),
  mySize (old.mySize),
  myPlaceMarkings (NULL)
{
  if (mySize)
    myPlaceMarkings = new class PlaceMarking[mySize];
  for (unsigned i = 0; i < mySize; i++)
    myPlaceMarkings[i] = old.myPlaceMarkings[i];
}

class GlobalMarking&
GlobalMarking::operator= (const class GlobalMarking& old)
{
  assert (&myNet == &old.myNet);
  assert (mySize == old.mySize);
  myErroneous = old.myErroneous;
  for (unsigned i = 0; i < mySize; i++)
    myPlaceMarkings[i] = old.myPlaceMarkings[i];
  return *this;
}

GlobalMarking::~GlobalMarking ()
{
  delete[] myPlaceMarkings;
}

class PlaceMarking&
GlobalMarking::operator[] (const class Place& place)
{
  assert (&myNet.getPlace (place.getIndex ()) == &place &&
	  place.getIndex () < mySize);
  return (*this)[place.getIndex ()];
}

#include "Printer.h"
void
GlobalMarking::display (const class Printer& printer) const
{
  printer.printRaw (isErroneous () ? "erroneous state " : "state ");
  printer.delimiter ('(')++;

  for (unsigned i = 0; i < mySize; i++) {
    if (!myPlaceMarkings[i].empty () &&
	!myPlaceMarkings[i].getPlace ()->isSuppressed ()) {
      printer.linebreak ();
      myPlaceMarkings[i].display (printer);
    }
  }

  printer--.linebreak ();
  printer.delimiter (')');
}
